/*
 * NextBorders.java	0.2
 *
 *	Copyright (c) 1998	Makoto Kato
 *	Copyright (c) 1997-1998	Sun Microsystems, Inc.
 */

package nextlf.plaf;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;

import java.awt.Component;
import java.awt.Insets;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.Color;
import java.awt.Graphics;
import java.io.Serializable;

/*
 * A NEXTSTEP/OPENSTEP implementation of Border.
 */

public class NextBorders
{
	/*
	 * A border class for header column of table
	 */
	public static class TableHeaderBorder extends AbstractBorder implements UIResource
	{
		protected Color	shadow;	// 80,80,80
		protected Color	highlight;	// 208, 208, 208
		protected Color	darkShadow;	// 40, 40, 40
		protected Color	moreDarkShadow = Color.black;	// 0, 0, 0


		public TableHeaderBorder( Color shadow, Color highlight, Color darkShadow )
		{
			this.shadow = shadow;
			this.highlight = highlight;
			this.darkShadow = darkShadow;
		}


		public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
		{
			g.setColor( highlight );
			g.drawLine( x, y, x + w - 2, y );
			g.drawLine( x, y, x, y + h - 2 );

			g.setColor( shadow );
			g.drawLine( x + w - 1, y, x + w - 1, y );
			g.drawLine( x, y + h - 1, x, y + h - 1 );

			g.setColor( darkShadow );
			g.drawLine( x + 1, y + h - 1, x + w - 2, y + h - 1 );

			g.setColor( moreDarkShadow );
			g.drawLine( x + w - 1, y + 1, x + w - 1, y + h - 1 );
		}


		public Insets getBorderInsets(Component c)
		{
			if (c instanceof JButton)
			{
				JButton	b = (JButton)c;
				return (b.isDefaultCapable()? new Insets(8, 8, 8, 8) : new Insets(2, 2, 2, 2));
			}

			return new Insets(2, 2, 2, 2);
		}
	}



	public static class BevelBorder extends AbstractBorder implements UIResource
	{
		private Color darkShadow = UIManager.getColor("controlShadow");
		private Color lightShadow = UIManager.getColor("controlLtHighlight");
		private boolean isRaised;


		public BevelBorder(boolean isRaised, Color darkShadow, Color lightShadow)
		{
			this.isRaised = isRaised;
			this.darkShadow = darkShadow;
			this.lightShadow = lightShadow;
		}


		public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
		{
			g.setColor((isRaised) ? lightShadow : darkShadow);
			g.drawLine(x, y, x + w - 1, y);           // top
			g.drawLine(x, y + h - 1, x, y + 1);         // left

			g.setColor((isRaised) ? darkShadow : lightShadow);
			g.drawLine(x + 1, y + h - 1, x + w - 1, y + h - 1);	// bottom
			g.drawLine(x + w - 1, y + h - 1, x + w-1, y + 1);	// right
		}


		public Insets getBorderInsets(Component c)
		{
			return new Insets(1, 1, 1, 1);
		}


		public boolean isOpaque(Component c)
		{
			return true;
		}
    }



	public static class FocusBorder extends AbstractBorder implements UIResource
	{
		private Color focus;
		private Color control;

		public FocusBorder(Color control, Color focus)
		{
			this.control = control;
			this.focus = focus;
		}

		public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
		{
			if (((JComponent)c).hasFocus())
			{
				g.setColor(focus);
				g.drawRect(x, y, w-1, h-1);
			}
			else
			{
				g.setColor(control);
				g.drawRect(x, y, w-1, h-1);
			}
		}

		public Insets getBorderInsets(Component c)
		{
			return new Insets(1,1,1,1);
		}

		final static Insets insets = new Insets(1, 1, 1, 1);
	}



	public static class ButtonBorder extends AbstractBorder implements UIResource
	{
		protected Color	focus = UIManager.getColor("activeCaptionBorder");	// do not use now
		protected Color	shadow = UIManager.getColor("controlShadow");	// 80, 80, 80
		protected Color	highlight = UIManager.getColor("controlHighlight");	// 255, 255, 255
		protected Color	darkShadow;	// 0, 0, 0


		public ButtonBorder(Color shadow, Color highlight, Color darkShadow, Color focus)
		{
			this.shadow = shadow;
            this.highlight = highlight;
            this.darkShadow = darkShadow;
            this.focus = focus;
		}


		public void paintBorder(Component c, Graphics g, int x, int y, int w, int h)
		{
			boolean	isPressed = false;
			boolean	hasFocus = false;
			boolean	canBeDefault = false;
			boolean	isDefault = false;

			if (c instanceof AbstractButton)
			{
				AbstractButton	b = (AbstractButton) c;
				ButtonModel	model = b.getModel();

				isPressed = (model.isArmed() && model.isPressed());
				hasFocus = (model.isArmed() && isPressed) || (b.isFocusPainted() && b.hasFocus());

				if (b instanceof JButton)
				{
					canBeDefault = ((JButton)b).isDefaultCapable();
					isDefault = ((JButton)b).isDefaultButton();
				}
			}

			g.setColor( isPressed ? darkShadow : highlight );
			g.drawLine( x, y, x + w - 2, y );
			g.drawLine( x, y, x, y + h - 2 );

			g.setColor( isPressed ? highlight : darkShadow );
			g.drawLine( x + w - 1, y, x + w - 1, y + h - 1 );
			g.drawLine( x, y + h - 1, x + w - 1, y + h - 1 );

			g.setColor( shadow );

			if( isPressed )
			{
				g.drawLine( x + 1, y + 1, x + 1, y + h - 3 );
				g.drawLine( x + 1, y + 1, x + w - 3, y + 1 );
			}
			else
			{
				g.drawLine( x + w - 2, y + 1, x + w - 2, y + h - 1 );
				g.drawLine( x + 1, y + h - 2, x + w - 2, y + h - 2 );
			}
		}


		public Insets getBorderInsets(Component c)
		{
			if (c instanceof JButton)
			{
				JButton	b = (JButton) c;

				return (b.isDefaultCapable()? new Insets(8, 8, 8, 8) : new Insets(2, 2, 2, 2));
			}

			return new Insets(2, 2, 2, 2);
		}
	}	// end ButtonBorder class implements



	public static class ToggleButtonBorder extends ButtonBorder
	{
		public ToggleButtonBorder(Color shadow, Color highlight, Color darkShadow, Color focus)
		{
			super(shadow, highlight, darkShadow, focus);
		}


		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			if (c instanceof AbstractButton)
			{
				AbstractButton	b = (AbstractButton) c;
				ButtonModel	model = b.getModel();

				if (model.isArmed() && model.isPressed() || model.isSelected())
					drawBezel(g, x, y, width, height, (model.isPressed() || model.isSelected()), b.isFocusPainted() && b.hasFocus(), shadow, highlight, darkShadow, focus);
				else
					drawBezel(g, x, y, width, height, false, b.isFocusPainted() && b.hasFocus(), shadow, highlight, darkShadow, focus);
			}
			else
			{
				drawBezel(g, x, y, width, height, false, false, shadow, highlight, darkShadow, focus);
			}
		}


		public Insets getBorderInsets(Component c)
		{
			return new Insets(2, 2, 3, 3);
		}
    }



	public static class MenuBarBorder extends ButtonBorder
	{
		public MenuBarBorder(Color shadow, Color highlight, Color darkShadow, Color focus)
		{
			super(shadow, highlight, darkShadow, focus);
		}


		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			JMenuBar	menuBar = (JMenuBar) c;

			if (menuBar.isBorderPainted() == true)
			{
				// this draws the MenuBar border
				Dimension size = menuBar.getSize();
				drawBezel(g,x,y,size.width,size.height,false,false, shadow, highlight, darkShadow, focus);
			}
		}


		public Insets getBorderInsets(Component c)
		{
			return new Insets(6,6,6,6);
		}
	}



	public static class FrameBorder extends AbstractBorder implements UIResource
	{
		JComponent	jcomp;
		Color	frameHighlight;
		Color	frameColor;
		Color	frameShadow;

		// The width of the border
		public final static int	BORDER_SIZE = 8;

		static Insets	insets = new Insets( BORDER_SIZE, BORDER_SIZE,  BORDER_SIZE, BORDER_SIZE);


		public FrameBorder(JComponent comp)
		{
			jcomp = comp;
		}


		public void setComponent(JComponent comp)
		{
			jcomp = comp;
		}


		public JComponent component()
		{
			return jcomp;
		}


		protected Color getFrameHighlight()
		{
			return frameHighlight;
		}


		protected Color getFrameColor()
		{
            return frameColor;
        }


		protected Color getFrameShadow()
		{
            return frameShadow;
        }


		public Insets getBorderInsets(Component c)
		{
            return insets;
        }


		/*
		 * Draws the FrameBorder's top border.
		 */
		protected boolean drawTopBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			g.setColor( Color.black );
			g.drawLine( x, y, x + width - 1, y );

			return true;
		}


        /** Draws the FrameBorder's left border.
          */
		protected boolean drawLeftBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			g.setColor( Color.black );
			g.drawLine( x, y, x, y + height - 1 );

			return true;
		}


        /** Draws the FrameBorder's right border.
          */
		protected boolean drawRightBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			g.setColor( Color.black );
			g.drawLine( x + width - 1, y, x + width - 1, y + height - 1 );

			return true;
		}


        /** Draws the FrameBorder's bottom border.
          */
		protected boolean drawBottomBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			Rectangle	borderRect;
			int	marginHeight, startY;

			borderRect = new Rectangle( 0, height - getBorderInsets(c).bottom, width, getBorderInsets(c).bottom);

			if (!g.getClipBounds().intersects(borderRect))
			{
				return false;
			}

			startY = height - getBorderInsets(c).bottom;

			g.setColor(frameShadow);
            g.drawLine(x + 1, height - 1, x + width - 1, height - 1);

			g.setColor(frameColor);
			g.fillRect(x + 2, startY + 1, width - 4, 6);

			g.setColor(frameHighlight);
			g.drawLine(x, startY, x + width - 1, startY);

			return true;
        }


        // Returns true if the associated component has focus.
		protected boolean isActiveFrame()
		{
			return jcomp.hasFocus();
		}


		/*
		 * Draws the FrameBorder in the given Rect.
		 */
		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			if (isActiveFrame())
				frameColor = UIManager.getColor("activeCaptionBorder");
			else
				frameColor = UIManager.getColor("inactiveCaptionBorder");

			frameHighlight = frameColor.brighter();
			frameShadow = frameColor.darker().darker();

			drawTopBorder(c, g, x, y, width, height);
			drawLeftBorder(c, g, x, y, width, height);
			drawRightBorder(c, g, x, y, width, height);
			drawBottomBorder(c, g, x, y, width, height);
		}
	}



	public static class InternalFrameBorder extends FrameBorder
	{
		JInternalFrame	frame;


		// The size of the bounding box for Motif frame corners.
		public final static int CORNER_SIZE = 32;


		/*
		 * Constructs an InternalFrameBorder for the InternalFrame
		 * <b>aFrame</b>.
		 */
		public InternalFrameBorder(JInternalFrame aFrame)
		{
			FrameBorder.super(aFrame);
			frame = aFrame;
		}


		/*
		 * Sets the InternalFrameBorder's InternalFrame.
		 */
		public void setFrame(JInternalFrame aFrame)
		{
			frame = aFrame;
		}


		/** Returns the InternalFrameBorder's InternalFrame.
          * @see #setFrame
          */
		public JInternalFrame frame()
		{
			return frame;
        }


		public int resizePartWidth()
		{
			if (!frame.isResizable())
			{
				return 0;
			}

			return FrameBorder.BORDER_SIZE;
		}


		/*
		 * Draws the InternalFrameBorder's top border.
		 */
		protected boolean drawTopBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			return true;
		}


		/*
		 * Draws the InternalFrameBorder's left border.
		 */
		protected boolean drawLeftBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			return true;
		}


		/*
		 * Draws the InternalFrameBorder's right border.
		 */
		protected boolean drawRightBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			return true;
		}


        /** Draws the InternalFrameBorder's bottom border.
          */
		protected boolean drawBottomBorder(Component c, Graphics g, int x, int y, int width, int height)
		{
			if (super.drawBottomBorder(c, g, x, y, width, height) && frame.isResizable())
			{
				int	startY = height - getBorderInsets(c).bottom;

				g.setColor(getFrameShadow());
				g.drawLine(CORNER_SIZE - 1, startY + 1, CORNER_SIZE - 1, height - 1);
				g.drawLine(width - CORNER_SIZE, startY + 1, width - CORNER_SIZE, height - 1);

				g.setColor(getFrameHighlight());
				g.drawLine(CORNER_SIZE, startY, CORNER_SIZE, height - 2);
				g.drawLine(width - CORNER_SIZE + 1, startY, width - CORNER_SIZE + 1, height - 2);

				return true;
			}

			return false;
		}


        // Returns true if the associated internal frame has focus.
		protected boolean isActiveFrame()
		{
			return frame.isSelected();
		}
	}


	public static void drawBezel(Graphics g, int x, int y, int w, int h, boolean isPressed, boolean hasFocus, Color shadow, Color highlight, Color darkShadow, Color focus)
	{
		Color	oldColor = g.getColor();
		g.translate(x, y);

		if (isPressed)
		{
			if (hasFocus)
			{
				g.setColor( focus );
				g.drawRect( 0, 0, w - 1, h - 1 );
			}

			g.setColor(shadow);         // inner border
			g.drawRect(1, 1, w-3, h-3);

			g.setColor(highlight);    // inner 3D border
			g.drawLine(2, h-3, w-3, h-3);
			g.drawLine(w-3, 2, w-3, h-4);
		}
		else
		{
			if (hasFocus)
			{
				g.setColor(focus);
				g.drawRect(0, 0, w-1, h-1);

				g.setColor(highlight);	// inner 3D border
				g.drawLine(1, 1, 1, h-3);
				g.drawLine(2, 1, w-4, 1);

				g.setColor(shadow);
				g.drawLine(2, h-3, w-3, h-3);
				g.drawLine(w-3, 1, w-3, h-4);

				g.setColor(darkShadow);        // black drop shadow  __|
				g.drawLine(1, h-2, w-2, h-2);
				g.drawLine(w-2, h-2, w-2, 1);
			}
			else
			{
				g.setColor(highlight);    // inner 3D border
				g.drawLine(1,1,1,h-3);
				g.drawLine(2,1,w-4,1);

				g.setColor(shadow);
				g.drawLine(2,h-3,w-3,h-3);
				g.drawLine(w-3,1,w-3,h-4);

				g.setColor(darkShadow);         // black drop shadow  __|
				g.drawLine(1,h-2,w-2,h-2);
				g.drawLine(w-2,h-2,w-2,0);
			}

			g.translate(-x, -y);
		}

		g.setColor(oldColor);
	}
}
