/*
 * NextButtonUI.java	0.1
 *
 *	Copyright (c) 1998	Makoto Kato
 *	Copyright (c) 1997-1998	Sun Microsystems, Inc.
 */

package nextlf.plaf;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.plaf.*;

/*
 * A NEXTSTEP/OPENSTEP implementation of ButtonUI.
 */
public class NextButtonUI extends BasicButtonUI
{
	private boolean defaults_initialized = false;
    protected Color selectColor;


	public static ComponentUI createUI(JComponent c)
    {
		return new NextButtonUI();
    }


    protected BasicButtonListener createButtonListener(AbstractButton b)
    {
		return new NextButtonListener(b);
    }


	public void installDefaults(AbstractButton b)
	{
		super.installDefaults(b);

		if(!defaults_initialized)
		{
			selectColor = UIManager.getColor(getPropertyPrefix() + "select");
			defaults_initialized = true;
		}

		b.setOpaque(false);
	}


	protected void uninstallDefaults(AbstractButton b)
	{
		super.uninstallDefaults(b);
		defaults_initialized = false;
	}


	protected Color getSelectColor()
	{
		return selectColor;
	}


	public void paint(Graphics g, JComponent c)
	{
		fillContentArea( g, (AbstractButton)c , c.getBackground() );
		super.paint(g,c);
	}


	protected void paintFocus(Graphics g, AbstractButton b, Rectangle viewRect, Rectangle textRect, Rectangle iconRect)
	{
		// focus painting is handled by the border
	}


	protected void paintButtonPressed(Graphics g, AbstractButton b)
	{
		fillContentArea( g, b , selectColor );
	}


	protected void fillContentArea( Graphics g, AbstractButton b, Color fillColor)
	{
		if (b.isContentAreaFilled())
		{
			Insets	margin = b.getMargin();
			Insets	insets = b.getInsets();
			Dimension	size = b.getSize();
			g.setColor(fillColor);
			g.fillRect(insets.left - margin.left,
			insets.top - margin.top,
			size.width - (insets.left-margin.left) - (insets.right - margin.right),
			size.height - (insets.top-margin.top) - (insets.bottom - margin.bottom));
		}
	}
}
