/*
 * NextComboBoxUI.java	0.1
 *
 *	Copyright (c) 1998	Makoto Kato
 *	Copyright (c) 1997-1998	Sun Microsystems, Inc.
 */

package nextlf.plaf;

import java.awt.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.*;
import java.io.Serializable;
import java.awt.event.*;

/*
 * A NEXTSTEP/OPENSTEP L&F implementation of ComboBoxUI.
 */
public class NextComboBoxUI extends BasicComboBoxUI implements Serializable
{
	Icon	arrowIcon;
	static final int	HORIZ_MARGIN = 3;


	public static ComponentUI createUI(JComponent c)
	{
		return new NextComboBoxUI();
	}


	public void installUI(JComponent c)
	{
		super.installUI(c);
		arrowIcon = new NextComboBoxArrowIcon();

 		Runnable initCode = new Runnable()
		{
			public void run()
			{
				if ( nextGetEditor() != null )
				{
					nextGetEditor().setBackground( UIManager.getColor( "text" ) );
				}
			}
		};

		SwingUtilities.invokeLater( initCode );
	}


	public Dimension getMinimumSize( JComponent c )
	{
		if ( !isMinimumSizeDirty )
		{
			return new Dimension( cachedMinimumSize );
		}

		Dimension	size;
		Insets	insets = getInsets();

		size = getDisplaySize();
		size.height += insets.top + insets.bottom;

		int	buttonSize = iconAreaWidth();

		size.width +=  insets.left + insets.right + buttonSize;

		cachedMinimumSize.setSize( size.width, size.height );
		isMinimumSizeDirty = false;

        return size;
    }


	protected ComboPopup createPopup()
	{
		return new NextComboPopup( comboBox );
	}


	protected void installComponents()
	{
		if ( comboBox.isEditable() )
		{
			addEditor();
		}

        comboBox.add( currentValuePane );
    }


    protected void uninstallComponents() {
        removeEditor();
        comboBox.removeAll();
    }


	public void paint(Graphics g, JComponent c)
	{
		boolean		hasFocus = comboBox.hasFocus();
		Rectangle	r;

		g.setColor( new Color( 160, 160, 160 ) );
		g.fillRect( 0, 0, c.getWidth(), c.getHeight() );

		if ( !comboBox.isEditable() )
		{
			r = rectangleForCurrentValue();
			paintCurrentValue(g,r,hasFocus);
		}

		r = rectangleForArrowIcon();
		arrowIcon.paintIcon(c,g,r.x,r.y);
	}


	public void paintCurrentValue(Graphics g,Rectangle bounds,boolean hasFocus)
	{
		ListCellRenderer	renderer = comboBox.getRenderer();
		Component	c;
		Dimension	d;

		c = renderer.getListCellRendererComponent(listBox, comboBox.getSelectedItem(), -1, false, false);
		c.setFont(comboBox.getFont());

		if ( comboBox.isEnabled() )
		{
			c.setForeground(comboBox.getForeground());
			c.setBackground(comboBox.getBackground());
		}
		else
		{
			c.setForeground( UIManager.getColor("ComboBox.disabledForeground") );
			c.setBackground( UIManager.getColor("ComboBox.disabledBackground") );
		}

		d  = c.getPreferredSize();
		currentValuePane.paintComponent(g,c,comboBox,bounds.x,bounds.y, bounds.width,d.height);
	}


	protected Rectangle rectangleForArrowIcon()
	{
		Rectangle	b = comboBox.getBounds();
		Border	border = comboBox.getBorder();
        Insets in = border.getBorderInsets(comboBox);
        b.x = in.left;
        b.y = in.top;
        b.width -= (in.left + in.right);
        b.height -= (in.top + in.bottom);

        b.x = b.x + b.width - HORIZ_MARGIN - arrowIcon.getIconWidth();
        b.y = b.y + (b.height - arrowIcon.getIconHeight()) / 2;
        b.width = arrowIcon.getIconWidth();
        b.height = arrowIcon.getIconHeight();
        return b;
    }

	protected Rectangle rectangleForCurrentValue()
	{
        int width = comboBox.getWidth();
        int height = comboBox.getHeight();
        Insets insets = getInsets();
        return new Rectangle(insets.left, insets.top,
                             (width - (insets.left + insets.right)) - iconAreaWidth(),
                             height - (insets.top + insets.bottom));
    }

    public int iconAreaWidth() {
        if ( comboBox.isEditable() )
            return arrowIcon.getIconWidth() + (2 * HORIZ_MARGIN);
        else
            return arrowIcon.getIconWidth() + (3 * HORIZ_MARGIN) + 2;
    }

    public void configureEditor() {
        super.configureEditor();
        editor.setBackground( UIManager.getColor( "text" ) );
    }

    protected LayoutManager createLayoutManager() {
        return new ComboBoxLayoutManager();
    }

    Component nextGetEditor() {
        return editor;
    }

    /**
     * This inner class is marked &quot;public&quot; due to a compiler bug.
     * This class should be treated as a &quot;protected&quot; inner class.
     * Instantiate it only within subclasses of <FooUI>.
     */
	public class ComboBoxLayoutManager extends BasicComboBoxUI.ComboBoxLayoutManager
	{
		public void layoutContainer(Container parent)
		{
			if ( nextGetEditor() != null )
			{
				Rectangle	cvb = rectangleForCurrentValue();
                cvb.x += 1;
                cvb.y += 1;
                cvb.width -= 1;
                cvb.height -= 2;
                nextGetEditor().setBounds(cvb);
            }
        }
    }

	static class NextComboBoxArrowIcon implements Icon, Serializable
	{
		private Color	lightShadow;
		private Color	darkShadow;
		private Color	fill;

		public NextComboBoxArrowIcon()
		{
        }


		public void paintIcon(Component c, Graphics g, int xo, int yo)
		{
			int width = getIconWidth();
			int height = getIconHeight();

			g.setColor( new Color( 80, 80, 80 ) );
			g.fillRect( xo + width - 13, yo + (height / 2), 9, 5 );
			g.drawLine( xo + width - 8, yo + height / 2 - 1, xo + width - 8, yo + height / 2 + 2 );
			g.drawLine( xo + width - 8, yo + height / 2 + 2, xo + width - 14, yo + height / 2 + 2 );

			g.setColor( new Color( 0, 0, 0 ) );
			g.drawLine( xo + width - 7, yo + height / 2 - 2, xo + width - 7, yo + height / 2 + 3 );
			g.drawLine( xo + width - 7, yo + height / 2 + 3, xo + width - 15, yo + height / 2 + 3 );

			g.setColor( new Color( 255, 255, 255 ) );
			g.drawLine( xo + width - 8, yo + height / 2 - 2, xo + width - 15, yo + height / 2 - 2 );
			g.drawLine( xo + width - 15, yo + height / 2 - 2, xo + width - 15, yo + height / 2 + 2 );

			g.setColor( new Color( 160, 160, 160 ) );
			g.fillRect( xo + width - 14, yo + (height / 2) - 1, 6, 3 );
		}


		public int getIconWidth()
		{
			return 15;
		}


		public int getIconHeight()
		{
			return 11;
		}
	}

    protected void selectNextPossibleValue() {
        super.selectNextPossibleValue();
    }

    protected void selectPreviousPossibleValue() {
        super.selectPreviousPossibleValue();
    }


	JComboBox nextGetComboBox()
	{
        return comboBox;
    }


	NextComboBoxUI nextGetUI()
	{
		return this;
	}

	protected void installKeyboardActions()
	{
		super.installKeyboardActions();

		AbstractAction	downAction = new AbstractAction()
		{
			public void actionPerformed( ActionEvent e )
			{
				nextGetUI().selectNextPossibleValue();
			}

			public boolean isEnabled()
			{
				return nextGetComboBox().isEnabled() && isPopupVisible(nextGetComboBox());
			}
        };

        nextGetComboBox().registerKeyboardAction( downAction, KeyStroke.getKeyStroke( KeyEvent.VK_DOWN, 0 ), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

        AbstractAction upAction = new AbstractAction() {
            public void actionPerformed( ActionEvent e ) {
                nextGetUI().selectPreviousPossibleValue();
            }
            public boolean isEnabled(){
                return nextGetComboBox().isEnabled() && isPopupVisible(nextGetComboBox());
            }
        };

		nextGetComboBox().registerKeyboardAction( upAction, KeyStroke.getKeyStroke( KeyEvent.VK_UP, 0 ), JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
    }

	protected void uninstallKeyboardActions()
	{
        super.uninstallKeyboardActions();
        comboBox.unregisterKeyboardAction(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN,0));
        comboBox.unregisterKeyboardAction(KeyStroke.getKeyStroke(KeyEvent.VK_UP,0));
    }



    /**
     * This inner class is marked &quot;public&quot; due to a compiler bug.
     * This class should be treated as a &quot;protected&quot; inner class.
     * Instantiate it only within subclasses of <FooUI>.
     */
	public class NextComboPopup extends BasicComboPopup
	{
		JComboBox	cBox;

		public NextComboPopup( JComboBox cBox )
		{
			super( cBox );

			this.cBox = cBox;
		}

		//
		// Don't view scrollbar
		//
		protected void configureScroller()
		{
			super.configureScroller();
			ScrollPaneLayout	layout = (ScrollPaneLayout) scroller.getLayout();
            layout.setVerticalScrollBarPolicy( ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER );
            layout.setHorizontalScrollBarPolicy( ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER );
            scroller.invalidate();
        }

		protected JScrollPane createScroller()
		{
			return new JScrollPane( list, ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER );
		}

		protected Rectangle computePopupBounds(int px,int py,int pw,int ph)
		{
			Rectangle	absBounds;
			boolean	inModalDialog = inModalDialog();
			Rectangle	bounds = cBox.getBounds();
			int	currentElementCount = cBox.getModel().getSize();
			absBounds = new Rectangle( 0, py, pw, ph );

			if ( currentElementCount > 0 )
			{
				Rectangle	cellBounds = list.getCellBounds( 0, 0 );

                Dimension	scrSize = Toolkit.getDefaultToolkit().getScreenSize();
                int	maxHeight = getPopupHeightForRowCount( currentElementCount );

                int	selectedIndex = cBox.getSelectedIndex();
                int	selectedLocation = getPopupHeightForRowCount( selectedIndex + 1 ) - 2;
                selectedLocation -= cellBounds.height;

				Point	top;
				Point	bottom;

				if ( inModalDialog )
				{
					Dialog	dialog = getDialog();
					top = SwingUtilities.convertPoint( dialog, 0, 0, cBox );
					bottom = SwingUtilities.convertPoint( dialog, 0, dialog.getSize().height - 1, cBox );
				}
				else
				{
					top = new Point( 0, 0 );
					SwingUtilities.convertPointFromScreen( top, cBox );
					bottom = new Point( 0, scrSize.height - 1 );
					SwingUtilities.convertPointFromScreen( bottom, cBox );
				}

                int theRest = maxHeight - selectedLocation;

                JViewport port = scroller.getViewport();
                Point viewPos = port.getViewPosition();

                // Test to see if the popup hits the top or the bottom of the screen.

                boolean scrollingNeededAtTop = top.y + selectedLocation > 0;
                boolean scrollingNeededAtBottom = theRest > bottom.y - 2;

				if ( scrollingNeededAtTop )
				{
                    absBounds.y = top.y + 1;

                    int newViewPos = selectedLocation + top.y;
                    port.setViewPosition( new Point( viewPos.x, newViewPos ) );

					if ( scrollingNeededAtBottom )
						absBounds.height = scrSize.height - 4;
                    else
                        absBounds.height = maxHeight - newViewPos;
                }
				else
				{
					absBounds.y = -selectedLocation + 1;
					port.setViewPosition( new Point( viewPos.x, 0 ) );

					if ( scrollingNeededAtBottom )
						absBounds.height = (bottom.y - absBounds.y) - 2;
					else
						absBounds.height = maxHeight;
				}
            }
			else
			{
				return super.computePopupBounds( px, py, pw, ph );
			}

            return absBounds;
        }


		public void show()
		{
			Dimension	popupSize = cBox.getSize();
			popupSize.setSize( popupSize.width, getPopupHeightForRowCount( cBox.getMaximumRowCount() ) );
			Rectangle	popupBounds = computePopupBounds( 0, cBox.getBounds().height, popupSize.width, popupSize.height);
            scroller.setMaximumSize( popupBounds.getSize() );
            scroller.setPreferredSize( popupBounds.getSize() );
            scroller.setMinimumSize( popupBounds.getSize() );
            list.invalidate();
            list.setSelectedIndex( cBox.getSelectedIndex() );
            list.ensureIndexIsVisible( list.getSelectedIndex() );

            setLightWeightPopupEnabled( cBox.isLightWeightPopupEnabled() );

			show( cBox, popupBounds.x, popupBounds.y );
        }


		private Dialog getDialog()
		{
			Container	parent;

			for ( parent = cBox.getParent() ; parent != null && !(parent instanceof Dialog) && !(parent instanceof Window) ; parent = parent.getParent() )
				;

			if ( parent instanceof Dialog )
				return (Dialog) parent;
			else
				return null;
		}


		private boolean inModalDialog()
		{
			return (getDialog() != null);
		}


		public void delegateFocus( MouseEvent e )
		{
		}
    }
}
