/*
 *	NextIconFactory.java	0.1
 *
 *	Copyright (c) 1998 Makoto Kato
 *	Copyright (c) 1997-1998 Sun Microsystems, Inc.
 */

package nextlf.plaf;

import javax.swing.*;

import javax.swing.plaf.UIResource;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Polygon;

import java.io.Serializable;

/**
 */
public class NextIconFactory implements Serializable
{
	private static Icon	checkBoxIcon;
	private static Icon	radioButtonIcon;
	private static Icon	menuItemCheckIcon;
	private static Icon	menuItemArrowIcon;
	private static Icon	menuArrowIcon;

	public static Icon getMenuItemCheckIcon()
	{
		if (menuItemCheckIcon == null)
			menuItemCheckIcon = new MenuItemCheckIcon();

		return menuItemCheckIcon;
	}


	public static Icon getMenuItemArrowIcon()
	{
		if (menuItemArrowIcon == null)
			menuItemArrowIcon = new MenuItemArrowIcon();

		return menuItemArrowIcon;
	}


	public static Icon getMenuArrowIcon()
	{
		if (menuArrowIcon == null)
			menuArrowIcon = new MenuArrowIcon();

		return menuArrowIcon;
	}


	public static Icon getCheckBoxIcon()
	{
		if (checkBoxIcon == null)
			checkBoxIcon = new CheckBoxIcon();

		return checkBoxIcon;
	}


	public static Icon getRadioButtonIcon()
	{
		if (radioButtonIcon == null)
			radioButtonIcon = new RadioButtonIcon();

		return radioButtonIcon;
	}



	private static class CheckBoxIcon implements Icon, UIResource, Serializable
	{
		final static int csize = 15;

		private	Color	control = UIManager.getColor("control");
		private Color	foreground = UIManager.getColor("CheckBox.foreground");
		private Color	shadow = UIManager.getColor("controlShadow");
		private Color	highlight = UIManager.getColor("controlHighlight");
		private Color	lightShadow = UIManager.getColor("controlLightShadow");


		public void paintIcon(Component c, Graphics g, int x, int y)
		{
			AbstractButton	b = (AbstractButton) c;
			ButtonModel	model = b.getModel();

			boolean	isPressed = model.isPressed();
			boolean	isArmed = model.isArmed();
			boolean isEnabled = model.isEnabled();
			boolean isSelected = model.isSelected();

			boolean checkToggleIn = ((isPressed && !isArmed && isSelected) || (isPressed && isArmed   && !isSelected));
			boolean uncheckToggleOut = ((isPressed && !isArmed && !isSelected) || (isPressed && isArmed && isSelected));
			boolean checkIn = (!isPressed  && isArmed && isSelected  || (!isPressed && !isArmed  && isSelected));

			x += 2;

			if (checkToggleIn)
			{
				// toggled from unchecked to checked
				drawCheckBezel(g,x,y,csize,true,false,false);
			}
			else if (uncheckToggleOut)
			{
				drawCheckBezel(g,x,y,csize,true,true,false);
			}
			else if (checkIn)
			{
				// show checked, unpressed state
				drawCheckBezel(g,x,y,csize,false,false,true);
			}
			else
			{ //  show unchecked state
				drawCheckBezelOut(g,x,y,csize);
			}
		}


		public int getIconWidth()
		{
			return csize;
		}


		public int getIconHeight()
		{
			return csize;
		}

		public void drawCheckBezelOut(Graphics g, int x, int y, int csize)
		{
			Color	controlShadow = UIManager.getColor("controlShadow");

			int	w = csize;
			int	h = csize;
			Color	oldColor = g.getColor();

			g.translate(x,y);
			g.setColor( Color.white );
			g.drawLine(0, 0, 0, h-2);
			g.drawLine(1, 0, w-2, 0);

			g.setColor( new Color( 0, 0, 0 ) );
			g.drawLine( 0, h - 1, w-1, h-1);
			g.drawLine( w - 1, h-1, w-1, 0);

			g.setColor( controlShadow );
			g.drawLine( w - 2, 1, w - 2, h - 2 );
			g.drawLine( 1, h - 2, w - 2, h - 2 );

			g.translate(-x,-y);
			g.setColor(oldColor);
		}


		public void drawCheckBezel(Graphics g, int x, int y, int csize, boolean shade, boolean out, boolean check)
		{
			int	w = csize;
			int	h = csize;
			Color	oldColor = g.getColor();

			g.translate(x, y);

			g.setColor( Color.white );
			g.drawLine(0, 0, 0, h-2);
			g.drawLine(1, 0, w-2, 0);

			g.setColor( Color.black );
			g.drawLine( 0, h - 1, w-1, h-1);
			g.drawLine( w - 1, h-1, w-1, 0);

			g.setColor( new Color( 80, 80, 80 ) );
			g.drawLine( w - 2, 1, w - 2, h - 2 );
			g.drawLine( 1, h - 2, w - 2, h - 2 );

			if (check)
			{
				// draw check

				g.setColor( Color.white );
				g.drawLine( csize - 4, 2, csize - 12, 10 );
				g.drawLine( csize - 12, 10, csize - 12, 5 );

			    g.setColor( new Color( 80, 80, 80 ) );
				g.drawLine( csize - 4, 4, csize - 11, 11 );
				g.drawLine( csize - 11, 8, csize - 10, 7 );

				g.setColor( Color.black );
				g.drawLine( csize - 4, 3, csize - 12, 11 );
				g.drawLine( csize - 11, 7, csize - 11, 5 );
			}

			g.translate(-x, -y);
			g.setColor(oldColor);
		}
	} // end class CheckBoxIcon


	private static class RadioButtonIcon implements Icon, UIResource, Serializable
	{
//		private Color dot = UIManager.getColor("activeCaptionBorder");
//		private Color highlight = UIManager.getColor("controlHighlight");
		private Color shadow = UIManager.getColor("controlShadow");

		public void paintIcon(Component c, Graphics g, int x, int y)
		{
			// fill interior
			AbstractButton	b = (AbstractButton) c;
			ButtonModel	model = b.getModel();

			int	w = getIconWidth();
			int	h = getIconHeight();

			// add pad so focus isn't smudged on the x
			x += 2;

			boolean	isPressed = model.isPressed();
			boolean	isArmed = model.isArmed();
			boolean	isEnabled = model.isEnabled();
			boolean	isSelected = model.isSelected();

			boolean	checkIn = ((isPressed && !isArmed && isSelected) || (isPressed && isArmed && !isSelected) || (!isPressed  && isArmed && isSelected  || (!isPressed && !isArmed  && isSelected)));

			// Draw
			g.setColor( shadow );
			g.drawLine( x + 2, y + 10, x + 2, y + 10 );
			g.drawLine( x + 2, y + 12, x + 2, y + 12 );
			g.drawLine( x + 1, y + 11, x + 1, y + 10 );
			g.drawLine( x, y + 9, x, y + 5 );
			g.drawLine( x + 1, y + 4, x + 1, y + 3 );
			g.drawLine( x + 2, y + 2, x + 2, y + 2 );
			g.drawLine( x + 3, y + 1, x + 4, y + 1 );
			g.drawLine( x + 5, y, x + 9, y );
			g.drawLine( x + 10, y + 1, x + 11, y + 1 );
			g.drawLine( x + 12, y + 2, x + 12, y + 2 );
			g.drawLine( x + 10, y + 2, x + 10, y + 2 );

			g.setColor( Color.black );
			g.drawLine( x + 1, y + 9, x + 1, y + 5 );
			g.drawLine( x + 2, y + 4, x + 2, y + 3 );
			g.drawLine( x + 3, y + 2, x + 4, y + 2 );
			g.drawLine( x + 5, y + 1, x + 9, y + 1);

			g.setColor( Color.white );
			g.drawLine( x + 14, y + 5, x + 14, y + 9 );
			g.drawLine( x + 13, y + 10, x + 13, y + 11 );
			g.drawLine( x + 12, y + 12, x + 12, y + 12 );
			g.drawLine( x + 11, y + 13, x + 10, y + 13 );
			g.drawLine( x + 9, y + 14, x + 5, y + 14 );
			g.drawLine( x + 4, y + 13, x + 4, y + 13 );

			if (checkIn)
			{
				g.drawLine( x + 6, y + 3, x + 9, y + 3 );
				g.drawLine( x + 4, y + 4, x + 11, y + 4 );
				g.drawLine( x + 4, y + 5, x + 11, y + 5 );
				g.drawLine( x + 3, y + 6, x + 12, y + 6 );
				g.drawLine( x + 3, y + 7, x + 12, y + 7 );
				g.drawLine( x + 3, y + 8, x + 12, y + 8 );
				g.drawLine( x + 3, y + 9, x + 12, y + 9 );
				g.drawLine( x + 4, y + 10, x + 11, y + 10 );
				g.drawLine( x + 4, y + 11, x + 11, y + 11 );
				g.drawLine( x + 6, y + 12, x + 9, y + 12 );
			}
		}


		public int getIconWidth()
		{
			return 13;
		}


		public int getIconHeight()
		{
			return 13;
		}
    } // end class RadioButtonIcon



	private static class MenuItemCheckIcon implements Icon, UIResource, Serializable
	{
		public void paintIcon(Component c,Graphics g, int x, int y)
		{
		}

		public int getIconWidth()
		{
			return 0;
		}

		public int getIconHeight()
		{
			return 0;
		}
	}  // end class MenuItemCheckIcon



	private static class MenuItemArrowIcon implements Icon, UIResource, Serializable
	{
		public void paintIcon(Component c,Graphics g, int x, int y)
		{
		}

		public int getIconWidth()
		{
			return 0;
		}

		public int getIconHeight() { return 0; }
    }  // end class MenuItemArrowIcon



	private static class MenuArrowIcon implements Icon, UIResource, Serializable
	{
		private Color focus = UIManager.getColor("windowBorder");
		private Color shadow = UIManager.getColor("controlShadow");
		private Color highlight = UIManager.getColor("controlHighlight");

		public void paintIcon(Component c, Graphics g, int x, int y)
		{
			AbstractButton b = (AbstractButton) c;
       ButtonModel model = b.getModel();

       int w = getIconWidth();
       int h = getIconHeight();


		Color	oldColor = g.getColor();
       // REMIND: this is a mess

		if (model.isSelected())
		{
			Polygon p = new Polygon();

			p.addPoint(x+1,y);
			p.addPoint(x+w-1,y+5);
			p.addPoint(x+1,y+h+1);

			g.setColor(focus);
			g.fillPolygon(p);
			g.drawLine(x+5,y+h-2,x+5,y+h-2);
	 g.drawLine(x+7,y+h-3,x+7,y+h-3);
	 g.setColor(shadow);
	 g.drawLine(x+1,y+1,x+1,y+h);
	 g.drawLine(x+2,y+1,x+2,y+1);
	 g.drawLine(x+4,y+2,x+4,y+2);
	 g.drawLine(x+6,y+3,x+6,y+3);
	 g.drawLine(x+8,y+4,x+8,y+5);
	 g.setColor(highlight);
	 g.drawLine(x+2,y+h,x+2,y+h);
	 g.drawLine(x+4,y+h-1,x+4,y+h-1);
	 g.drawLine(x+6,y+h-2,x+6,y+h-2);
	 g.drawLine(x+8,y+h-3,x+8,y+h-4);
       }
       else {
	 g.setColor(highlight);
	 g.drawLine(x+1,y+1,x+1,y+h);
	 g.drawLine(x+2,y+1,x+2,y+h-2);
	 g.fillRect(x+3,y+2,2,2);
	 g.fillRect(x+5,y+3,2,2);
	 g.fillRect(x+7,y+4,2,2);
	 g.setColor(shadow);
	 g.drawLine(x+2,y+h-1,x+2,y+h);
	 g.fillRect(x+3,y+h-2,2,2);
	 g.fillRect(x+5,y+h-3,2,2);
	 g.fillRect(x+7,y+h-4,2,2);
	 g.setColor(oldColor);
       }

     }
     public int getIconWidth() { return 10; }
     public int getIconHeight() { return 10; }
    } // End class MenuArrowIcon
}
