/*
 * NextRadioButtonUI.java	0.1
 *
 *	Copyright (c) 1998 Makoto Kato <m_kato@ga2.so-net.ne.jp>
 *	Copyright (c) 1997-1998 Sun Microsystems, Inc.
 */

package nextlf.plaf;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.BasicRadioButtonUI;
import javax.swing.plaf.basic.BasicGraphicsUtils;
import javax.swing.plaf.*;
import java.awt.*;

/*
 * A NEXTSTEP/OPENSTEP L&F implementation of RadioButtonUI.
 */
public class NextRadioButtonUI extends BasicRadioButtonUI {

    private static final NextRadioButtonUI nextRadioButtonUI = new NextRadioButtonUI();

    protected Color focusColor;

    private boolean defaults_initialized = false;


	public static ComponentUI createUI(JComponent c)
	{
		return nextRadioButtonUI;
	}


	public void installDefaults(AbstractButton b)
	{
		super.installDefaults(b);
		if(!defaults_initialized)
		{
			focusColor = UIManager.getColor(getPropertyPrefix() + "focus");
			defaults_initialized = true;
		}
    }


	protected void uninstallDefaults(AbstractButton b)
	{
		super.uninstallDefaults(b);
		defaults_initialized = false;
	}


	protected Color getFocusColor()
	{
		return focusColor;
	}


	public synchronized void paint(Graphics g, JComponent c)
	{
		AbstractButton b = (AbstractButton) c;
		ButtonModel model = b.getModel();

		Dimension size = c.getSize();

		int	w = size.width;
        int h = size.height;

		Font f = c.getFont();
        g.setFont(f);
        FontMetrics fm = g.getFontMetrics();

		Rectangle viewRect = new Rectangle(size);
        Rectangle iconRect = new Rectangle();
        Rectangle textRect = new Rectangle();

        Icon altIcon = b.getIcon();
        Icon selectedIcon = null;
        Icon disabledIcon = null;

		String	text = SwingUtilities.layoutCompoundLabel( c, fm, b.getText(), altIcon != null ? altIcon : getDefaultIcon(), b.getVerticalAlignment(), b.getHorizontalAlignment(), b.getVerticalTextPosition(), b.getHorizontalTextPosition(), viewRect, iconRect, textRect, getDefaultTextIconGap(b) );

        // fill background
		if(c.isOpaque())
		{
            g.setColor(b.getBackground());
            g.fillRect(0,0, size.width, size.height);
        }

        // Paint the radio button
		if(altIcon != null)
		{
			if(!model.isEnabled())
			{
				altIcon = b.getDisabledIcon();
			}
			else if(model.isPressed() && model.isArmed())
			{
				altIcon = b.getPressedIcon();

				if(altIcon == null)
				{
					// Use selected icon
					altIcon = b.getSelectedIcon();
				}
			}
			else if(model.isSelected())
			{
				if(b.isRolloverEnabled() && model.isRollover())
				{
					altIcon = (Icon) b.getRolloverSelectedIcon();

					if (altIcon == null)
						altIcon = (Icon) b.getSelectedIcon();
				}
				else
				{
					altIcon = (Icon) b.getSelectedIcon();
				}
			}
			else if(b.isRolloverEnabled() && model.isRollover())
			{
				altIcon = (Icon) b.getRolloverIcon();
			}

			if(altIcon == null)
				altIcon = b.getIcon();

			altIcon.paintIcon(c, g, iconRect.x, iconRect.y);
		}
		else
		{
			getDefaultIcon().paintIcon(c, g, iconRect.x, iconRect.y);
		}

        // Draw the Text
		if(text != null)
		{
			if(model.isEnabled())
			{
                // *** paint the text normally
                g.setColor(b.getForeground());
                BasicGraphicsUtils.drawString(g,text,model.getMnemonic(),
                                              textRect.x, textRect.y + fm.getAscent());
            } else {
                // *** paint the text disabled
                g.setColor(b.getBackground().darker());
                BasicGraphicsUtils.drawString(g,text,model.getMnemonic(),
                                              textRect.x,
                                              textRect.y + fm.getAscent());
            }
            if(b.hasFocus() && b.isFocusPainted() &&
               textRect.width > 0 && textRect.height > 0 ) {
                paintFocus(g,textRect,size);
            }
        }
    }


    protected void paintFocus(Graphics g, Rectangle t, Dimension d)
    {
		g.setColor(getFocusColor());
		g.drawRect(0,0,d.width-1,d.height-1);
	}
}
