/*
 *	NextScrollBarButton.java	0.1
 *
 *	Copyright (c) 1998 Makoto Kato <m_kato@ga2.so-net.ne.jp>
 *	Copyright (c) 1997-1998 Sun Microsystems, Inc.
 */

package nextlf.plaf;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.BasicArrowButton;

import java.awt.*;
import java.awt.event.*;

/*
 * A NEXTSTEP/OPENSTEP L&F implementation of scorllbar's button.
 */
public class NextScrollBarButton extends BasicArrowButton
{
	private Color	buttonShadow = UIManager.getColor("controlShadow");
	private Icon	buttonIcon;


	public NextScrollBarButton(int direction)
	{
		super(direction);

		switch (direction)
		{
		case NORTH:
		case SOUTH:
		case EAST:
		case WEST:
			this.direction = direction;
			break;

		default:
			throw new IllegalArgumentException("invalid direction");
		}

		setRequestFocusEnabled(false);
		setOpaque(true);
		setBackground(UIManager.getColor("ScrollBar.background"));
		setForeground(UIManager.getColor("ScrollBar.foreground"));
	}


	public Dimension getPreferredSize()
	{
		switch (direction) {
		case NORTH:
		case SOUTH:
			return new Dimension(16, 17);

		case EAST:
		case WEST:
		default:
			return new Dimension(17, 16);
		}
	}


	public Dimension getMinimumSize()
	{
		return getPreferredSize();
	}


    public Dimension getMaximumSize() {
	return getPreferredSize();
    }

    public boolean isFocusTraversable() {
	return false;
    }


	public void paint(Graphics g)
	{
		int	w = getWidth();
		int	h = getHeight();

		if (isOpaque())
		{
			g.setColor(getBackground());
			g.fillRect(0, 0, w, h);
		}

		boolean	isPressed = getModel().isPressed();
		Color	fill = getBackground();

		int	cx = w / 2;
		int	cy = h / 2;
		int	s = Math.min(w, h);

		int	offset = 0;

		if( isPressed )
		{
			offset = 1;
			g.setColor( Color.black );
			g.drawLine( 0, 0, s - 2, 0 );
			g.drawLine( 0, 0, 0, s - 2 );

			g.setColor( Color.white );
			g.drawLine( s - 1, 0, s - 1, s - 1 );
			g.drawLine( 0, s - 1, s - 1, s - 1 );

			g.setColor( new Color( 160, 160, 160 ) );
			g.fillRect( 1, 1, s - 2, s - 2 );

			g.setColor( buttonShadow );
			g.drawLine( 1, 1, s - 2, 1 );
			g.drawLine( 1, 1, 1, s - 2 );
		}
		else
		{
			g.setColor( Color.white );
			g.drawLine( 0, 0, s - 2, 0 );
			g.drawLine( 0, 0, 0, s - 2 );

			g.setColor( Color.black );
			g.drawLine( s - 1, 0, s - 1, s - 1 );
			g.drawLine( 0, s - 1, s - 1, s - 1 );

			g.setColor( new Color( 160, 160, 160 ) );
			g.fillRect( 1, 1, s - 3, s - 3 );

			g.setColor( buttonShadow );
			g.drawLine( s - 2, 0, s - 2, s - 2 );
			g.drawLine( 1, s - 2, s - 2, s - 2 );
		}

		switch (direction)
		{
		case NORTH:
			g.setColor( buttonShadow );
			g.drawLine( 3 + offset, 11 + offset, 7 + offset, 3 + offset );
			g.drawLine( 7 + offset, 3 + offset, 11 + offset, 11 + offset );

			g.setColor( Color.black );
			g.drawLine( 4 + offset, 11 + offset, 4 + offset, 10 + offset );
			g.drawLine( 5 + offset, 11 + offset, 5 + offset, 8 + offset );
			g.drawLine( 6 + offset, 11 + offset, 6 + offset, 6 + offset );
			g.drawLine( 7 + offset, 11 + offset, 7 + offset, 4 + offset );
			g.drawLine( 8 + offset, 11 + offset, 8 + offset, 6 + offset );
			g.drawLine( 9 + offset, 11 + offset, 9 + offset, 8 + offset );
			g.drawLine( 10 + offset, 11 + offset, 10 + offset, 10 + offset );

			break;

		case SOUTH:
			g.setColor( buttonShadow );
			g.drawLine( 3 + offset, 3 + offset, 7 + offset, 11 + offset );
			g.drawLine( 7 + offset, 11 + offset, 11 + offset, 3 + offset );

			g.setColor( Color.black );
			g.drawLine( 4 + offset, 3 + offset, 4 + offset, 4 + offset );
			g.drawLine( 5 + offset, 3 + offset, 5 + offset, 6 + offset );
			g.drawLine( 6 + offset, 3 + offset, 6 + offset, 8 + offset );
			g.drawLine( 7 + offset, 3 + offset, 7 + offset, 10 + offset );
			g.drawLine( 8 + offset, 3 + offset, 8 + offset, 8 + offset );
			g.drawLine( 9 + offset, 3 + offset, 9 + offset, 6 + offset );
			g.drawLine( 10 + offset, 3 + offset, 10 + offset, 4 + offset );

			break;

		case EAST:
			g.setColor( buttonShadow );
			g.drawLine( 3 + offset, 3 + offset, 11 + offset, 7 + offset );
			g.drawLine( 11 + offset, 7 + offset, 3 + offset, 11 + offset );

			g.setColor( Color.black );
			g.drawLine( 3 + offset, 4 + offset, 4 + offset, 4 + offset );
			g.drawLine( 3 + offset, 5 + offset, 6 + offset, 5 + offset );
			g.drawLine( 3 + offset, 6 + offset, 8 + offset, 6 + offset );
			g.drawLine( 3 + offset, 7 + offset, 10 + offset, 7 + offset );
			g.drawLine( 3 + offset, 8 + offset, 8 + offset, 8 + offset );
			g.drawLine( 3 + offset, 9 + offset, 6 + offset, 9 + offset );
			g.drawLine( 3 + offset, 10 + offset, 4 + offset, 10 + offset );

			break;

		case WEST:
			g.setColor( buttonShadow );
			g.drawLine( 11 + offset, 3 + offset, 3 + offset, 7 + offset );
			g.drawLine( 3 + offset, 7 + offset, 11 + offset, 11 + offset );

			g.setColor( Color.black );
			g.drawLine( 11 + offset, 4 + offset, 10 + offset, 4 + offset );
			g.drawLine( 11 + offset, 5 + offset, 8 + offset, 5 + offset );
			g.drawLine( 11 + offset, 6 + offset, 6 + offset, 6 + offset );
			g.drawLine( 11 + offset, 7 + offset, 4 + offset, 7 + offset );
			g.drawLine( 11 + offset, 8 + offset, 6 + offset, 8 + offset );
			g.drawLine( 11 + offset, 9 + offset, 8 + offset, 9 + offset );
			g.drawLine( 11 + offset, 10 + offset, 10 + offset, 10 + offset );

			break;
		}
	}
}
