/*
 *  NextScrollBarUI.java	0.2
 *
 *	Copyright (c) 1998 Makoto Kato <m_kato@ga2.so-net.ne.jp>
 *	Copyright (c) 1997-1998 Sun Microsystems, Inc.
 */
package nextlf.plaf;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.BasicScrollBarUI;
import javax.swing.plaf.basic.BasicGraphicsUtils;

import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.event.*;


/*
 * A NEXTSTEP/OPENSTEP L&F implementation of ScrollBarUI.
 */
public class NextScrollBarUI extends BasicScrollBarUI
{
	private static Color	thumbShadowColor = UIManager.getColor("controlShadow");


	public static ComponentUI createUI(JComponent c)
	{
		return new NextScrollBarUI();
	}


	public Dimension getPreferredSize(JComponent c)
	{
		Insets	insets = c.getInsets();
		int	dx = insets.left + insets.right;
		int	dy = insets.top + insets.bottom;

		return (scrollbar.getOrientation() == JScrollBar.VERTICAL) ? new Dimension(dx + 16, dy + 48 ) : new Dimension(dx + 48, dy + 16 );
	}


	protected JButton createDecreaseButton(int orientation)
	{
		return new NextScrollBarButton(orientation);
	}


	protected JButton createIncreaseButton(int orientation)
	{
		return new NextScrollBarButton(orientation);
	}


	protected TrackListener createTrackListener()
	{
		return new NextTrackListener();
	}


	public void paintTrack(Graphics g, JComponent c, Rectangle trackBounds)
	{
		NextScrollTrackPattern	pattern = new NextScrollTrackPattern( trackBounds.width, trackBounds.height );
		pattern.paintIcon( c, g, trackBounds.x, trackBounds.y );
    }


	public void paintThumb(Graphics g, JComponent c, Rectangle thumbBounds)
	{
		if(thumbBounds.isEmpty() || !scrollbar.isEnabled())
			return;

		int	w = thumbBounds.width;
		int	h = thumbBounds.height;

		g.translate(thumbBounds.x, thumbBounds.y);

		g.setColor( thumbColor );
		g.fillRect(1, 1, w - 3, h - 3);

		g.setColor( thumbHighlightColor );
		g.drawLine(0, 0, 0, h - 2);
		g.drawLine(1, 0, w - 2, 0);

		g.setColor( thumbShadowColor );
		g.drawLine( w - 2, 1, w - 2, h - 2 );
		g.drawLine( 1, h - 2, w - 2, h - 2 );

		g.setColor( Color.black );
		g.drawLine( w - 1, 0, w - 1, h - 1 );
		g.drawLine( 0, h - 1, w - 1, h - 1 );

		// Draw Thumb
		g.setColor( thumbShadowColor );
		g.drawLine( w / 2 - 3, h / 2 - 2, w / 2 - 2, h / 2 - 3 );
		g.drawLine( w / 2 - 2, h / 2 + 2, w / 2 - 2, h / 2 - 1 );
		g.drawLine( w / 2 - 1, h / 2 - 1, w / 2 - 1, h / 2 - 2 );
		g.drawLine( w / 2, h / 2 - 2, w / 2 + 2, h / 2 - 2 );

		g.setColor( Color.black );
		g.drawLine( w / 2 - 3, h / 2 + 1, w / 2 - 3, h  / 2 - 1 );
		g.drawLine( w / 2 - 3, h / 2 - 1, w / 2 - 1, h / 2 - 3 );
		g.drawLine( w / 2, h / 2 - 3, w / 2 + 1, h / 2 - 3 );

		g.setColor( thumbHighlightColor );
		g.drawLine( w / 2 + 1, h / 2, w/ 2 + 2, h / 2 );
		g.drawLine( w / 2, h / 2 + 1, w/ 2 + 2, h / 2 + 1 );
		g.drawLine( w / 2, h / 2 + 2, w/ 2 + 1, h / 2 + 2 );

		g.translate(-thumbBounds.x, -thumbBounds.y);
	}


	protected void layoutVScrollbar(JScrollBar sb)
	{
        Dimension	sbSize = sb.getSize();
        Insets	sbInsets = sb.getInsets();

		/*
		 * Width and left edge of the buttons and thumb.
		 */
		int	itemW = sbSize.width - (sbInsets.left + sbInsets.right);
		int	itemX = sbInsets.left;

		/* Nominal locations of the buttons, assuming their preferred
		 * size will fit.
		 */
		int	incrButtonH = incrButton.getPreferredSize().height;
		int	incrButtonY = sbSize.height - (sbInsets.bottom + incrButtonH);

		int	decrButtonH = decrButton.getPreferredSize().height;
		int	decrButtonY = sbSize.height - (sbInsets.bottom + incrButtonH + decrButtonH);

        /* The thumb must fit within the height left over after we
		 * subtract the preferredSize of the buttons and the insets.
		 */

		int	sbInsetsH = sbInsets.top + sbInsets.bottom;
		int	sbButtonsH = decrButtonH + incrButtonH;
		float	trackH = sbSize.height - (sbInsetsH + sbButtonsH);

        /* Compute the height and origin of the thumb.   The case
		 * where the thumb is at the bottom edge is handled specially
		 * to avoid numerical problems in computing thumbY.  Enforce
		 * the thumbs min/max dimensions.  If the thumb doesn't
		 * fit in the track (trackH) we'll hide it later.
		 */

		float	min = sb.getMinimum();
		float	extent = sb.getVisibleAmount();
		float	range = sb.getMaximum() - min;
		float	value = sb.getValue();

		int	thumbH = (range <= 0) ? getMaximumThumbSize().height : (int)(trackH * (extent / range));
		thumbH = Math.max(thumbH, getMinimumThumbSize().height);
		thumbH = Math.min(thumbH, getMaximumThumbSize().height);

		int thumbY = decrButtonY - thumbH;

		if (sb.getValue() < (sb.getMaximum() - sb.getVisibleAmount()))
		{
			float thumbRange = trackH - thumbH;
			thumbY = (int)(0.5f + (thumbRange * ((value - min) / (range - extent))));
			thumbY += sbInsets.top;
		}

        /* If the buttons don't fit, allocate half of the available
		 * space to each and move the lower one (incrButton) down.
		 */
		int sbAvailButtonH = (sbSize.height - sbInsetsH);

		if (sbAvailButtonH < sbButtonsH)
		{
            incrButtonH = decrButtonH = sbAvailButtonH / 2;
		}

		decrButton.setBounds(itemX, decrButtonY, itemW, decrButtonH);
		incrButton.setBounds(itemX, incrButtonY, itemW, incrButtonH);

		/* Update the trackRect field.
		 */
		int itrackY = sbInsets.top;
		int itrackH = sbSize.height - (incrButtonH + decrButtonH);
		trackRect.setBounds( itemX, itrackY, itemW, itrackH );

		/* If the thumb isn't going to fit, zero it's bounds.  Otherwise
		 * make sure it fits between the buttons.  Note that setting the
		 * thumbs bounds will cause a repaint.
		 */

		if( thumbH >= (int) trackH )
		{
			setThumbBounds(0, 0, 0, 0);
		}
		else
		{
			if ((thumbY + thumbH) > decrButtonY)
				thumbY = decrButtonY - thumbH;

			if (thumbY  < sbInsets.top)
				thumbY = sbInsets.top + 1;

			setThumbBounds(itemX, thumbY, itemW, thumbH);
		}
    }


	protected void layoutHScrollbar(JScrollBar sb)
    {
		Dimension	sbSize = sb.getSize();
		Insets	sbInsets = sb.getInsets();

		/* Height and top edge of the buttons and thumb.
		 */
		int itemH = sbSize.height - (sbInsets.top + sbInsets.bottom);
		int itemY = sbInsets.top;

		/* Nominal locations of the buttons, assuming their preferred
		 * size will fit.
		 */
		int	decrButtonW = decrButton.getPreferredSize().width;
		int	decrButtonX = sbInsets.left;

        int incrButtonW = incrButton.getPreferredSize().width;
        int incrButtonX = sbInsets.left + decrButtonW;

        /* The thumb must fit within the width left over after we
		 * subtract the preferredSize of the buttons and the insets.
		 */
		int	sbInsetsW = sbInsets.left + sbInsets.right;
		int	sbButtonsW = decrButtonW + incrButtonW;
		float	trackW = sbSize.width - (sbInsetsW + sbButtonsW);

        /* Compute the width and origin of the thumb.  Enforce
		 * the thumbs min/max dimensions.  The case where the thumb
		 * is at the right edge is handled specially to avoid numerical
		 * problems in computing thumbX.  If the thumb doesn't
		 * fit in the track (trackH) we'll hide it later.
		 */

		float	min = sb.getMinimum();
        float	extent = sb.getVisibleAmount();
        float	range = sb.getMaximum() - min;
        float	value = sb.getValue();

		int	thumbW = (range <= 0) ? getMaximumThumbSize().width : (int)(trackW * (extent / range));
		thumbW = Math.max(thumbW, getMinimumThumbSize().width);
		thumbW = Math.min(thumbW, getMaximumThumbSize().width);

		int thumbX = decrButtonX + sbSize.width;

		if (sb.getValue() < (sb.getMaximum() - sb.getVisibleAmount()))
		{
			float thumbRange = trackW - thumbW;
			thumbX = (int)(0.5f + (thumbRange * ((value - min) / (range - extent))));
			thumbX +=  incrButtonX + incrButtonW;
		}

        /* If the buttons don't fit, allocate half of the available
         * space to each and move the right one (incrButton) over.
         */

		int	sbAvailButtonW = (sbSize.width - sbInsetsW);

		if (sbAvailButtonW < sbButtonsW)
		{
			incrButtonW = decrButtonW = sbAvailButtonW / 2;
        }

		decrButton.setBounds(decrButtonX, itemY, decrButtonW, itemH);
		incrButton.setBounds(incrButtonX, itemY, incrButtonW, itemH);

		/*
		 * Update the trackRect field.
		 */

		int	itrackX = incrButtonX + incrButtonW;
		int	itrackW = sbSize.width - (incrButtonW + decrButtonW);
		trackRect.setBounds(itrackX, itemY, itrackW, itemH);

		/*
		 * Make sure the thumb fits between the buttons.  Note
		 * that setting the thumbs bounds causes a repaint.
		 */

		if (thumbW >= (int) trackW)
		{
			setThumbBounds(0, 0, 0, 0);
		}
		else
		{
			if (thumbX + thumbW > incrButtonX + incrButtonW + sbSize.width )
				thumbX = incrButtonX + incrButtonW + sbSize.width - thumbW;

			if (thumbX  < incrButtonX + incrButtonW)
				thumbX = incrButtonX + incrButtonW + 1;

			setThumbBounds(thumbX, itemY, thumbW, itemH);
		}
	}




	protected class NextTrackListener extends TrackListener
	{
		public void mousePressed(MouseEvent e)
		{
			if(!scrollbar.isEnabled())
				return;

			scrollbar.setValueIsAdjusting(true);

			currentMouseX = e.getX();
			currentMouseY = e.getY();

			// Clicked in the Thumb area?
			if(getThumbBounds().contains(currentMouseX, currentMouseY))
			{
				switch (scrollbar.getOrientation())
				{
				case JScrollBar.VERTICAL:
					offset = currentMouseY - getThumbBounds().y;
					break;

				case JScrollBar.HORIZONTAL:
					offset = currentMouseX - getThumbBounds().x;
					break;
				}

				isDragging = true;
				return;
			}

			isDragging = false;

			Dimension	sbSize = scrollbar.getSize();
			int	direction = +1;

            switch (scrollbar.getOrientation())
            {
			case JScrollBar.VERTICAL:
				if (getThumbBounds().isEmpty())
				{
					int	scrollbarCenter = sbSize.height / 2;
					direction = (currentMouseY < scrollbarCenter) ? -1 : +1;
				}
				else
				{
					int	thumbY = getThumbBounds().y;
					direction = (currentMouseY < thumbY) ? -1 : +1;
				}

				break;

			case JScrollBar.HORIZONTAL:
				if (getThumbBounds().isEmpty())
				{
					int	scrollbarCenter = sbSize.width / 2;
					direction = (currentMouseX < scrollbarCenter) ? -1 : +1;
				}
				else
				{
					int	thumbX = getThumbBounds().x;
					direction = (currentMouseX < thumbX) ? -1 : +1;
				}

				break;
			}

			scrollByBlock(direction);

			if(!getThumbBounds().contains(currentMouseX, currentMouseY))
			{
				scrollTimer.stop();
				scrollListener.setDirection(direction);
				scrollListener.setScrollByBlock(true);
				scrollTimer.start();
			}
		}


		public void mouseDragged(MouseEvent e)
		{
			if(!scrollbar.isEnabled() || !isDragging)
				return;

			BoundedRangeModel	model = scrollbar.getModel();
			Rectangle	thumbR = getThumbBounds();
			float	trackLength;
			int	thumbMin, thumbMax, thumbPos;

			if (scrollbar.getOrientation() == JScrollBar.VERTICAL)
			{
				thumbMin = incrButton.getY() + incrButton.getHeight() - scrollbar.getSize().height;
				thumbMax = decrButton.getY() - getThumbBounds().height;

				thumbPos = Math.min(thumbMax, Math.max(thumbMin, (e.getY() - offset)));
				setThumbBounds(thumbR.x, thumbPos, thumbR.width, thumbR.height);
				trackLength = getTrackBounds().height;
			}
			else
			{
				thumbMin = incrButton.getX() + incrButton.getWidth();
				thumbMax = decrButton.getX() + scrollbar.getSize().width - getThumbBounds().width;
				thumbPos = Math.min(thumbMax, Math.max(thumbMin, (e.getX() - offset)));
				setThumbBounds(thumbPos, thumbR.y, thumbR.width, thumbR.height);
				trackLength = getTrackBounds().width;
			}

			if (thumbPos == thumbMax)
			{
				scrollbar.setValue(model.getMaximum() - model.getExtent());
			}
			else
			{
				float	valueMax = model.getMaximum() - model.getExtent();
				float	valueRange = valueMax - model.getMinimum();
				float	thumbValue = thumbPos - thumbMin;
				float	thumbRange = thumbMax - thumbMin;
				int	value = (int)(0.5 + ((thumbValue / thumbRange) * valueRange));
				scrollbar.setValue(value + model.getMinimum());
			}
		}

		public void mouseMoved(MouseEvent e)
		{
		}
	}
}
