/*
 *	NextScrollPaneUI.java	0.2
 *
 *	Copyright (c) 1998 Makoto Kato <m_kato@ga2.so-net.ne.jp>
 *	Copyright (c) 1997-1998 Sun Microsystems, Inc.
 */

package nextlf.plaf;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.BasicScrollPaneUI;


/**
 * A NEXTSTEP/OPENSTEP L&F implementation of ScrollPaneUI.
 */

public class NextScrollPaneUI extends BasicScrollPaneUI
{
    private final static Border vsbMarginBorder = new EmptyBorder(0, 4, 0, 0);
    private final static Border hsbMarginBorder = new EmptyBorder(4, 0, 0, 0);

	private Border vsbBorder;
	private Border hsbBorder;


	protected void installDefaults(JScrollPane scrollpane)
	{
		super.installDefaults(scrollpane);

		JScrollBar vsb = scrollpane.getVerticalScrollBar();

		if (vsb != null)
		{
			vsbBorder = new CompoundBorder(vsbMarginBorder, vsb.getBorder());
			vsb.setBorder(vsbBorder);
		}

		JScrollBar hsb = scrollpane.getHorizontalScrollBar();
		if (hsb != null)
		{
			hsbBorder = new CompoundBorder(hsbMarginBorder, hsb.getBorder());
			hsb.setBorder(hsbBorder);
		}
	}


	protected void uninstallDefaults(JScrollPane c)
	{
		super.uninstallDefaults(c);

		JScrollBar vsb = scrollpane.getVerticalScrollBar();

		if (vsb != null)
		{
			if (vsb.getBorder() == vsbBorder)
				vsb.setBorder(null);

			vsbBorder = null;
		}


		JScrollBar hsb = scrollpane.getHorizontalScrollBar();

		if (hsb != null)
		{
			if (hsb.getBorder() == hsbBorder)
				hsb.setBorder(null);

			hsbBorder = null;
		}
	}


	public static ComponentUI createUI(JComponent x)
	{
		return new NextScrollPaneUI();
	}
}

