/*
 *	NextSplitPaneDivider.java	0.2
 *
 *	Copyright (c) 1998 Makoto Kato <m_kato@ga2.so-net.ne.jp>
 *	Copyright (c) 1997-1998 Sun Microsystems, Inc.
 */

package nextlf.plaf;

import java.awt.*;
import javax.swing.JSplitPane;
import javax.swing.UIManager;
import javax.swing.plaf.basic.BasicSplitPaneUI;
import javax.swing.plaf.basic.BasicSplitPaneDivider;


public class NextSplitPaneDivider extends BasicSplitPaneDivider
{
	public static final int	minimumThumbSize = 6;
	public static final int	defaultDividerSize = 10;

	protected static final int	pad = 6;

	protected int	hThumbWidth = 8;
	protected int	hThumbHeight = 8;
	protected int	vThumbWidth = 8;
	protected int	vThumbHeight = 8;


	public NextSplitPaneDivider(BasicSplitPaneUI ui)
	{
		super(ui);

		setDividerSize(hThumbWidth + pad);
	}


	public void setDividerSize(int newSize)
	{
		if (newSize < pad + minimumThumbSize)
		{
			setDividerSize(pad + minimumThumbSize);
		}
		else
		{
			vThumbHeight = hThumbWidth = newSize - pad;

			super.setDividerSize(newSize);
		}
	}


	public void paint(Graphics g)
	{
		Dimension	size = getSize();

        // fill
		g.setColor(getBackground());
		g.fillRect(0, 0, size.width, size.height);

		/*
		 * draw thumb
		 */
		g.setColor( new Color( 80, 80, 80 ) );
		g.drawLine( size.width / 2 - 3, size.height / 2 - 2, size.width / 2 - 2, size.height / 2 - 3 );
		g.drawLine( size.width / 2 - 2, size.height / 2 + 2, size.width / 2 - 2, size.height / 2 - 1 );
		g.drawLine( size.width / 2 - 1, size.height / 2 - 1, size.width / 2 - 1, size.height / 2 - 2 );
		g.drawLine( size.width / 2, size.height / 2 - 2, size.width / 2 + 2, size.height / 2 - 2 );

		g.setColor( Color.black );
		g.drawLine( size.width / 2 - 3, size.height / 2 + 1, size.width / 2 - 3, size.height  / 2 - 1 );
		g.drawLine( size.width / 2 - 3, size.height / 2 - 1, size.width / 2 - 1, size.height / 2 - 3 );
		g.drawLine( size.width / 2, size.height / 2 - 3, size.width / 2 + 1, size.height / 2 - 3 );

		g.setColor( Color.white );
		g.drawLine( size.width / 2 + 1, size.height / 2, size.width / 2 + 2, size.height / 2 );
		g.drawLine( size.width / 2, size.height / 2 + 1, size.width / 2 + 2, size.height / 2 + 1 );
		g.drawLine( size.width / 2, size.height / 2 + 2, size.width / 2 + 1, size.height / 2 + 2 );

		super.paint(g);
    }


	/*
	 * The minimums size is the same as the preferredSize
	 */
	public Dimension getMinimumSize()
	{
		return getPreferredSize();
	}
}
